/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.clientserverapi.topology.exception;

/**
 * An error occurs attaching a container to a domain.
 * 
 * @author Christophe DENEUX - Linagora
 */
public class AttachContainerException extends TopologyServiceException {

    private static final long serialVersionUID = 1771724545178817580L;

    private static final String MESSAGE_TEMPLATE_WITHOUT_CURRENT_CONTAINER = "Can't attach the current container to the domain '%s' using the container '%s': %s";

    private static final String MESSAGE_TEMPLATE_WITH_CURRENT_CONTAINER = "Can't attach the current container '%s' to the domain '%s' using the container '%s': %s";

    /**
     * Name of the domain to attach to
     */
    private final String domainName;

    /**
     * Name of the container to used as entry point in the domain
     */
    private final String containerName;

    /**
     * @param domainName
     *            The target domain in which the current container must be attached
     * @param currentContainerName
     *            The name of the current container, the one that is moving.
     * @param containerName
     *            The name of the container to used as entry point in the target domain to attach the current container
     * @param cause
     *            The error cause
     */
    public AttachContainerException(final String domainName, final String currentContainerName,
            final String containerName, final String cause) {
        super(String.format(MESSAGE_TEMPLATE_WITH_CURRENT_CONTAINER, currentContainerName,
                domainName == null ? "not identified" : domainName, containerName, cause));
        this.domainName = domainName;
        this.containerName = containerName;
    }

    /**
     * @param domainName
     *            The target domain in which the current container must be attached
     * @param currentContainerName
     *            The name of the current container, the one that is moving.
     * @param containerName
     *            The name of the container to used as entry point in the target domain to attach the current container
     * @param cause
     *            The error cause
     */
    public AttachContainerException(final String domainName, final String currentContainerName,
            final String containerName, final Throwable cause) {
        this(domainName, currentContainerName, containerName, cause.getMessage(), cause);
    }

    /**
     * @param domainName
     *            The target domain in which the current container must be attached
     * @param containerName
     *            The name of the container to used as entry point in the target domain to attach the current container
     * @param cause
     *            The error cause
     */
    public AttachContainerException(final String domainName, final String containerName, final Throwable cause) {
        super(String.format(MESSAGE_TEMPLATE_WITHOUT_CURRENT_CONTAINER,
                domainName == null ? "not identified" : domainName, containerName, cause.getMessage()), cause);
        this.domainName = domainName;
        this.containerName = containerName;
    }

    /**
     * @param domainName
     *            The target domain in which the current container must be attached
     * @param currentContainerName
     *            The name of the current container, the one that is moving.
     * @param containerName
     *            The name of the container to used as entry point in the target domain to attach the current container
     * @param causeMsg
     *            The error cause message
     * @param cause
     *            The error cause stack trace
     */
    public AttachContainerException(final String domainName, final String currentContainerName,
            final String containerName, final String causeMsg, final Throwable cause) {
        super(String.format(MESSAGE_TEMPLATE_WITH_CURRENT_CONTAINER, currentContainerName,
                domainName == null ? "not identified" : domainName, containerName, causeMsg), cause);
        this.domainName = domainName;
        this.containerName = containerName;
    }

    /**
     * @return Name of the container to used as entry point in the target domain
     */
    public String getContainerName() {
        return this.containerName;
    }

    /**
     * @return Name of the domain to attach to
     */
    public String getDomainName() {
        return domainName;
    }

}
